﻿using System;
using System.Collections.Generic;
using System.Data;
using MySql.Data.MySqlClient;

namespace DBShardTools.Core
{
    public class MySqlPersister : PersisterBase
    {
        public override IDbConnection GetConnection()
        {
            return new MySqlConnection(ConnectionString);
        }

        public override IConnectionStringBuilder GetConnectionStringBuilder()
        {
            return new MySqlConnectionStringBuilder();
        }
        public override string Name
        {
            get
            {
                return "MySql";
            }
        }

        public override string Description
        {
            get { return "MySql"; }
        }

        
        public override string GetCreateTableDdl(string tableName)
        {
            string createTableDdl = string.Empty;
            string commandText = string.Format("show create table {0}", tableName);
            IDbConnection conn;
            IDbCommand comm;
            IDataReader reader = null;
            using (conn = GetConnection())
            {
                comm = conn.CreateCommand();
                conn.Open();
                comm.CommandText = commandText;
                try
                {
                    reader = comm.ExecuteReader();
                    reader.Read();
                    createTableDdl = reader[1] as string;
                }
                catch (Exception ex)
                {
                    //throw ex;
                }
                finally
                {
                    if (reader != null)
                    reader.Close();
                }
            }
            if (!string.IsNullOrEmpty(createTableDdl))
                createTableDdl = createTableDdl.Replace("\n","\r\n");
            return createTableDdl;
        }

        public override string GetAlterTableDdl(string tableName)
        {
            return string.Format("ALTER TABLE `{0}`", tableName);
        }

        public override string GetDropTableDdl(string tableName)
        {
            return string.Format("DROP TABLE `{0}`", tableName);
        }

        public override string[] GetTables()
        {
            List<string> tables = new List<string>();
            string commandText = string.Format("show TABLES");
            IDbConnection conn;
            IDbCommand comm;
            IDataReader reader = null;
            using (conn = GetConnection())
            {
                comm = conn.CreateCommand();
                conn.Open();
                comm.CommandText = commandText;
                try
                {
                    reader = comm.ExecuteReader();
                    while(reader.Read())
                        tables.Add(reader[0] as string);
                }
                catch (Exception ex)
                {
                    //throw ex;
                }
                finally
                {
                    if (reader != null)
                        reader.Close();
                }
            }
            return tables.ToArray();
        }

        private int _port;
        public override int Port
        {
            get
            {
                if (_port == 0)
                    _port = 3306;

                return _port;
            }
            set { _port = value; }
        }

        private string _userName;
        public override string UserName
        {
            get
            {
                if (string.IsNullOrEmpty(_userName))
                    _userName = "root";
                return _userName;
            }
            set { _userName = value; }
        }
    }
}
