﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Windows.Forms;
using DBShardTools.Core;
using Microsoft.Practices.Unity;

namespace DBShardTools.UI
{
    
    public sealed partial class ConfigForm : Form
    {
        private IPersister _persister;
        private readonly IUnityContainer _container;
        private readonly IConfigService _configService;
        public ConfigForm(IUnityContainer container,IConfigService configService)
        {
            InitializeComponent();


            _container = container;
            _configService = configService;

            InitChkbSelectConnectionState();
            DataBindPersister();
        }
        private void DataBindPersister()
        {
            IEnumerable<IPersister> persisters = _container.ResolveAll<IPersister>();
            List<Item> list = persisters.Select(persister => new Item(persister.Description, persister.Name)).ToList();
            ddlPersister.DataSource = list;
            if (ddlPersister.Items.Count > 0)
                ddlPersister.SelectedIndex = 0;
        }
        private void InitChkbSelectConnectionState()
        {
            IConnection[] connections = _configService.GetConnections();
            chkbSelectConnection.Enabled = (connections != null && connections.Length > 0);
        }
        private void DdlPersisterSelectedIndexChanged(object sender, EventArgs e)
        {
            if (ddlPersister.SelectedIndex != -1)
            {
                _persister = _container.Resolve<IPersister>(ddlPersister.SelectedValue.ToString());
                SetDefaultConnectionSetting(_persister);
            }
        }
        private void SetDefaultConnectionSetting(IPersister persister)
        {
            txtHost.Text = persister.Host;
            txtPort.Text = persister.Port.ToString();
            txtUserName.Text = persister.UserName;
            ddlConnections.Text = string.Format("{0}@{1}", persister.UserName,persister.Host);
        }

        private void BtnConfirmClick(object sender, EventArgs e)
        {
            UpdatePersister(_persister);
            _container.RegisterInstance(_persister);
            SaveConnection();
            this.DialogResult = DialogResult.OK;
        }
        private void UpdatePersister(IPersister persister)
        {
            persister.Host = txtHost.Text.Trim();
            persister.Port = int.Parse(txtPort.Text.Trim());
            persister.UserName = txtUserName.Text.Trim();
            persister.Password = txtPassword.Text.Trim();
            persister.Database = txtDatabase.Text.Trim();
        }

        private void BtnTestClick(object sender, EventArgs e)
        {
            new Thread(() =>
                {
                    SetTestButtonState(false);
                    UpdatePersister(_persister);
                    if (_persister.TestConnection())
                        MessageBox.Show("连接成功！", "系统消息", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    else
                        MessageBox.Show("连接失败！", "系统消息", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    SetTestButtonState(true);
                }).Start();
        }
        private void SetTestButtonState(bool enabled)
        {
            if (btnTest.InvokeRequired)
            {
                this.Invoke(new Action(() =>
                {
                    btnTest.Enabled = enabled;
                }));
            }
            else
                btnTest.Enabled = enabled;
        }
        private void SaveConnection()
        {
            _configService.SaveConnection(ddlConnections.Text.Trim(), ddlPersister.SelectedValue.ToString(), txtHost.Text.Trim(), int.Parse(txtPort.Text.Trim()), txtDatabase.Text.Trim(), txtUserName.Text.Trim(), txtPassword.Text.Trim());
        }

        private void ChkbSelectConnectionCheckStateChanged(object sender, EventArgs e)
        {
            if (chkbSelectConnection.Checked)
            {
                ddlConnections.DropDownStyle = ComboBoxStyle.DropDownList;
                DataBindConnections();
            }
            else
            {
                ddlConnections.DropDownStyle = ComboBoxStyle.Simple;
            }
        }
        private void DataBindConnections()
        {
            IConnection[] connections = _configService.GetConnections();
            ddlConnections.DisplayMember = "Name";
            ddlConnections.ValueMember = "Name";
            ddlConnections.DataSource = connections;

        }

        private void DdlConnectionsSelectedIndexChanged(object sender, EventArgs e)
        {
            if (ddlConnections.SelectedIndex != -1)
            {
                IConnection connection = _configService.GetConnection(ddlConnections.SelectedValue.ToString());

                ddlPersister.SelectedValue = connection.DatabaseType;
                txtHost.Text = connection.Host;
                txtPort.Text = connection.Port.ToString();
                txtDatabase.Text = connection.Database;
                txtUserName.Text = connection.UserName;
                txtPassword.Text = connection.Password;

                UpdatePersister(_persister);
            }
        }
    }
}
